package example.dds.gateway;

import org.jboss.soa.esb.ConfigurationException;
import org.jboss.soa.esb.Service;
import org.jboss.soa.esb.client.ServiceInvoker;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.listeners.ListenerTagNames;
import org.jboss.soa.esb.listeners.lifecycle.AbstractThreadedManagedLifecycle;
import org.jboss.soa.esb.listeners.lifecycle.ManagedLifecycleException;
import org.jboss.soa.esb.listeners.message.MessageDeliverException;
import org.jboss.soa.esb.message.Message;
import org.jboss.soa.esb.message.format.MessageFactory;

import DDS.*;

import Test.*;

public class OpenSpliceGateway extends AbstractThreadedManagedLifecycle {	
    private ConfigTree listenerConfig;
	
	private Service service;
	private ServiceInvoker serviceInvoker;
	
    private DomainParticipantFactory dpf;
    private DomainParticipant parentDP;
    private Topic testMessageTopic;
    private Subscriber testSubscriber;
    private DataReader parentReader;
    
    private TestItemTypeSupport itemTS;
    private TestItemDataReader itemReader;
    private TestItemSeqHolder msgSeq;
    private SampleInfoSeqHolder infoSeq;

    /* QosPolicy holders */
    private TopicQosHolder reliableTopicQos;
    private SubscriberQosHolder subQos;
    
    private String domain = null;
    private String partitionName = new String("test");
    private String itemTypeName;
    
    public OpenSpliceGateway(final ConfigTree config) throws ConfigurationException {
        super(config);
        this.listenerConfig = config;

        String serviceCategory = listenerConfig.getRequiredAttribute(ListenerTagNames.TARGET_SERVICE_CATEGORY_TAG);
        String serviceName = listenerConfig.getRequiredAttribute(ListenerTagNames.TARGET_SERVICE_NAME_TAG);
 
        service = new Service(serviceCategory, serviceName);
    }

	@Override
	protected void doInitialise() throws ManagedLifecycleException {
		try {	
			serviceInvoker = new ServiceInvoker(service);
			msgSeq = new TestItemSeqHolder();
		    infoSeq = new SampleInfoSeqHolder();
		    
		    reliableTopicQos = new TopicQosHolder();
		    subQos = new SubscriberQosHolder();
	        
		    dpf = DomainParticipantFactory.get_instance();
		    parentDP = dpf.create_participant(domain, PARTICIPANT_QOS_DEFAULT.value, null, STATUS_MASK_NONE.value);
		    
		    itemTS = new TestItemTypeSupport();
		    itemTypeName = itemTS.get_type_name();
		    itemTS.register_type(parentDP, itemTypeName);
		    
		    parentDP.get_default_topic_qos(reliableTopicQos);
	        reliableTopicQos.value.reliability.kind = ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
	        parentDP.set_default_topic_qos(reliableTopicQos.value);
	        
	        testMessageTopic = parentDP.create_topic("Test_TestMessage", itemTypeName, reliableTopicQos.value, null, STATUS_MASK_NONE.value);
	        
	        parentDP.get_default_subscriber_qos(subQos);
	        subQos.value.partition.name = new String[1];
	        subQos.value.partition.name[0] = partitionName;

	        testSubscriber = parentDP.create_subscriber(subQos.value, null, STATUS_MASK_NONE.value);
	        parentReader = testSubscriber.create_datareader(testMessageTopic, DATAREADER_QOS_USE_TOPIC_QOS.value, null, STATUS_MASK_NONE.value);
	        itemReader = TestItemDataReaderHelper.narrow(parentReader);
		} catch (MessageDeliverException mde) {
			throw new ManagedLifecycleException("Unexpected message delivery exception.", mde);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void doThreadedDestroy() throws ManagedLifecycleException {
		testSubscriber.delete_datareader(itemReader);
		parentDP.delete_subscriber(testSubscriber);
		parentDP.delete_topic(testMessageTopic);
		dpf.delete_participant(parentDP);
	}

	@Override
	protected void doRun() {
		while (isRunning()) {
			itemReader.take(msgSeq, infoSeq, LENGTH_UNLIMITED.value, ANY_SAMPLE_STATE.value, ANY_VIEW_STATE.value, ALIVE_INSTANCE_STATE.value);
			
            for (TestItem currentItem : msgSeq.value)
			{
				try {
					Message esbMessage = MessageFactory.getInstance().getMessage();
			        esbMessage.getBody().add(currentItem);
			        serviceInvoker.deliverAsync(esbMessage);
		        } catch (Exception e) {
		    	    System.out.println("Unexpected MessageDeliverException: " + e.getMessage());
		        }
			}
			
			itemReader.return_loan(msgSeq, infoSeq);
	        msgSeq.value = null;
	        infoSeq.value = null;
	        
	        try {
	            Thread.sleep(100);
	        } catch (InterruptedException e) {
	            e.printStackTrace();
	        }
		}
	}
}
