package example.dds.testclient;

import DDS.*;
import Test.*;

public class OpenSpliceTestClient {

	private DomainParticipantFactory dpf;
	private DomainParticipant participant;
	private Topic testMessageTopic;
	private Publisher testPublisher;
    
    private TestItemTypeSupport itemTS;
	private TestItemDataWriter parentWriter;
	
	private TestItem testItem;

	private TopicQosHolder reliableTopicQos = new TopicQosHolder();
	private TopicQosHolder settingTopicQos = new TopicQosHolder();
	private PublisherQosHolder pubQos = new PublisherQosHolder();

    private String domain = null;
    private String partitionName = new String("test");
    
    private long handle;
    
    public OpenSpliceTestClient() {
    	dpf = DomainParticipantFactory.get_instance();
	    participant = dpf.create_participant(domain, PARTICIPANT_QOS_DEFAULT.value, null, STATUS_MASK_NONE.value);
	    itemTS = new TestItemTypeSupport();
	    itemTS.register_type(participant, itemTS.get_type_name());
	    
	    participant.get_default_topic_qos(reliableTopicQos);
	    reliableTopicQos.value.reliability.kind = ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
	    
        participant.set_default_topic_qos(reliableTopicQos.value);
        testMessageTopic = participant.create_topic("Test_TestMessage", itemTS.get_type_name(), reliableTopicQos.value, null, STATUS_MASK_NONE.value);
        
        participant.get_default_topic_qos(settingTopicQos);
        settingTopicQos.value.durability.kind = DurabilityQosPolicyKind.TRANSIENT_DURABILITY_QOS;
        participant.get_default_publisher_qos(pubQos);
        pubQos.value.partition.name = new String[1];
	    pubQos.value.partition.name[0] = partitionName;
	    
	    testPublisher = participant.create_publisher(pubQos.value, null, STATUS_MASK_NONE.value);
	    
	    parentWriter = TestItemDataWriterHelper.narrow(testPublisher.create_datawriter(testMessageTopic, DATAWRITER_QOS_USE_TOPIC_QOS.value, null, STATUS_MASK_NONE.value));
    }
    
    public void sendAlert() {
	    
    	testItem = new TestItem();
	    handle = parentWriter.register_instance(testItem);
	    testItem.id = 100;
	    
	    int status = parentWriter.write(testItem, handle);
	    
	    System.out.println("Return status of the write: " + status);
    }
    
    public void shutdown() {
    	parentWriter.dispose(testItem, handle);
    	parentWriter.unregister_instance(testItem, handle);
    	testPublisher.delete_datawriter(parentWriter);
    	participant.delete_publisher(testPublisher);
    	participant.delete_topic(testMessageTopic);
    	dpf.delete_participant(participant);
    }

	public static void main(String args[]) {
		OpenSpliceTestClient ostc = new OpenSpliceTestClient();
		
		ostc.sendAlert();
		
		ostc.shutdown();
	}
}
